@@ -1,5 +1,7 @@
 #[macro_use]
 extern crate derive_is_enum_variant;
+#[macro_use]
+extern crate derive_new;
 
 mod errors;
 mod lex;
@@ -9,9 +11,11 @@ mod scope;
 mod shapes;
 mod signature;
 
-pub use lex::{block, lex, LiteBlock, LiteCommand, LiteGroup, LitePipeline};
+pub use lex::lexer::{lex, parse_block};
+pub use lex::tokens::{LiteBlock, LiteCommand, LiteGroup, LitePipeline};
 pub use parse::{classify_block, garbage, parse, parse_full_column_path, parse_math_expression};
 pub use path::expand_ndots;
+pub use path::expand_path;
 pub use scope::ParserScope;
 pub use shapes::shapes;
 pub use signature::{Signature, SignatureRegistry};
